/**
-- Annotate Geometry
---------------------------------------------------------------------------------------------------
-- Description: 
-- -- MacroScript that display selected Geometry informations in the active Viewport.
---------------------------------------------------------------------------------------------------
-- Written by: hrrasto
-- eMail: hrrasto@azet.sk
-- www: http://hrrasto.php5.sk/maxScripts
-- Based on: Tape Annotate v1.01 by Paul Bertino published on: http://www.scriptspot.com/3ds-max/scripts/tape-annotate
---------------------------------------------------------------------------------------------------
-- Installation:
-- -- Place at: C:\Users\%USER%\AppData\Local\Autodesk\3dsMax\20xx - 64bit\ENU\usermacros\
-- -- (Re)Start 3dsMax
-- -- Goto: Customize > Customize User Interface > Quads Panel > Category: Tools > Action: Annotate Geometry
-- -- Drag "Annotate Geometry" and Drop in "Quad display"
---------------------------------------------------------------------------------------------------
-- Revisions:
-- -- 29.01.2020 - Iplementation of function getObjectDims obj -  from: http://www.scriptspot.com/forums/3ds-max/scripts-wanted/select-by-dimension-x-y-or-z
-- -- 10.08.2020 - Added Diameter as secondary parameter of Radius for standard and extended revolution primitives
-- -- 09.10.2020 - Added Material info in line 98
-- -- 09.10.2020 - Added Configuration options for Common object properties
-- -- 20.10.2020 - Added Rotation parameter
-- -- 21.10.2020 - Added Scale parameter
---------------------------------------------------------------------------------------------------
-- Need to add:
-- -- 03.10.2020 - Modifiers stack list for any geometry
-- -- 09.10.2020 - Configuration interface for displayed items
--
*/
macroScript AnnotateGeometry
category: "Tools" --Edit the category as you see fit
buttonText: "Annotate Geometry"
tooltip: "Annotate Geometry"
(
	local Version = 4.3

	global AnnotateGeometryEnabled = false
	unRegisterRedrawViewsCallback showGeometryInfo

	/** Annotation text colors */
	local geometryColor = color 196 196 254
	local shapeColor = color 63 254 254
	
	/** Config file Tools-AnnotateGeometry.ini */
	local macrosFolder = GetDir #userMacros
	local configFilename = (macrosFolder + "\Tools-AnnotateGeometry.ini")
	try (
		format "\nAnnotate Geometry %\n" Version
		format "Config Filename: %\n" configFilename
		local ViewName = getINISetting configFilename "Common" "Name" as booleanClass
		local viewPos = getINISetting configFilename "Common" "Position" as booleanClass
		local viewRot = getINISetting configFilename "Common" "Rotation" as booleanClass
		local viewScale = getINISetting configFilename "Common" "Scale" as booleanClass
		local viewMaterial = getINISetting configFilename "Common" "Material" as booleanClass
		local viewLayer = getINISetting configFilename "Common" "Layer" as booleanClass
		local viewClass = getINISetting configFilename "Common" "Class" as booleanClass
	) catch (
		format "Config File is Damaged or missing...\n"
		local ViewName = true
		local viewPos = true
		local viewRot = true
		local viewScale = true
		local viewMaterial = true
		local viewLayer = true
		local viewClass = true
	)

	/* getObjectDims
	  * from: http://www.scriptspot.com/forums/3ds-max/scripts-wanted/select-by-dimension-x-y-or-z */
	fn getObjectDims obj = (
		local objLocalBBox = nodeLocalBoundingBox obj
		--tm = obj.objecttransform
		--bmin = objLocalBBox[1]*(inverse tm) 
		--bmax = objLocalBBox[2]*(inverse tm)
		local bbox_size = objLocalBBox[2] - objLocalBBox[1]
		
		local Xdim = units.formatValue(abs bbox_size[1])
		local Ydim = units.formatValue(abs bbox_size[2])
		local Zdim = units.formatValue(abs bbox_size[3])
		
		return "\nDimensions:\n   X: "+Xdim+"\n   Y: "+Ydim+"\n   Z: "+Zdim
	)
	
	fn showGeometryInfo = (
		gw.setTransform(Matrix3 1)
		-- Objects -- whole Scene
		-- Selection -- selectionSet = selection as array --create array from selected objects
		for oBJ in selection where not oBJ.isHidden and not oBJ.isTarget and (SuperClassOf oBJ == GeometryClass or SuperClassOf oBJ == Shape) do (
			
			local oBJtext = ""

			/** Common object properties */
			/** oBJ as string will display the class, name and position inline, use for debug */
			--oBJtext += "\noBJ: "+(oBJ as string)

			if (ViewName == true) then (
				oBJtext += oBJ.name
			)
			if (viewPos == true) then (
				oBJtext += "\nPosition: "+(oBJ.pos as string)
			)
			if (viewRot == true) then (
				oBJtext += "\nRotation: "+(oBJ.rotation.x_rotation as string)+","+(oBJ.rotation.y_rotation as string)+","+(oBJ.rotation.z_rotation as string)
				--local rot = oBJ.rotation --as eulerAngles
				--oBJtext += "\nRotation: "+(rot.x as string)+" "+(rot.y as string)+" "+(rot.z as string)
				--oBJtext += "\nRotation:"+(rot as angleaxis) as string
				--oBJtext += "\nRotation:"+(rot.angle as string) 
			)
			if (viewScale == true) then (
				oBJtext += "\nScale: "+(oBJ.scale as string)
			)
			if (viewMaterial == true) then (
				oBJtext += "\nMaterial: "+(oBJ.material as string)
			)
			if (viewLayer == true) then (
				oBJtext += "\nLayer: "+oBJ.layer.name
			)
			if (viewClass == true) then (
				oBJtext += "\nClass: "+(ClassOf oBJ as string)
			)

			case SuperClassOf oBJ of (
				GeometryClass:(
					case ClassOf oBJ of (
						-- Standard Primitives
						Box: (
							oBJtext += "\nLength: "+(oBJ.length as string)+"\nWidth: "+(oBJ.width as string)+"\nHeight: "+(oBJ.height as string)
						)
						Plane: (
							oBJtext += "\nLength: "+(oBJ.length as string)+"\nWidth: "+(oBJ.width as string)
						)
						Cylinder: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nHeight: "+(oBJ.height as string)
						)
						Cone: (
							local diameter1 = (oBJ.radius1 * 2)
							local diameter2 = (oBJ.radius2 * 2)
							oBJtext += "\nRadius1: "+(oBJ.radius1 as string)+"\nRadius2: "+(oBJ.radius2 as string)+"\nDiameter1: "+(diameter1 as string)+"\nDiameter2: "+(diameter2 as string)+"\nHeight: "+(oBJ.height as string)
						)
						Tube: (
							local diameter1 = (oBJ.radius1 * 2)
							local diameter2 = (oBJ.radius2 * 2)
							oBJtext += "\nRadius1: "+(oBJ.radius1 as string)+"\nRadius2: "+(oBJ.radius2 as string)+"\nDiameter1: "+(diameter1 as string)+"\nDiameter2: "+(diameter2 as string)+"\nHeight: "+(oBJ.height as string)
						)
						Sphere: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nSegments: "+(oBJ.segs as string)
						)
						GeoSphere: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nSegments: "+(oBJ.segs as string)
						)
						Pyramid: (
							oBJtext += "\nWidth: "+(oBJ.width as string)+"\nDepth: "+(oBJ.depth as string)+"\nHeight: "+(oBJ.height as string)
						)
						Torus: (
							local diameter1 = (oBJ.radius1 * 2)
							local diameter2 = (oBJ.radius2 * 2)
							oBJtext += "\nRadius1: "+(oBJ.radius1 as string)+"\nRadius2: "+(oBJ.radius2 as string)+"\nDiameter1: "+(diameter1 as string)+"\nDiameter2: "+(diameter2 as string)
						)
						Teapot: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nSegments: "+(oBJ.segs as string)
						)
						
						--Extended Primitives
						ChamferBox: (
							oBJtext += "\nLength: "+(oBJ.length as string)+"\nWidth: "+(oBJ.width as string)+"\nHeight: "+(oBJ.height as string)+"\nFillet: "+(oBJ.fillet as string)
						)
						ChamferCyl: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nHeight: "+(oBJ.height as string)+"\nFillet: "+(oBJ.fillet as string)
						)
						Capsule: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nHeight: "+(oBJ.height as string)
						)
						OilTank: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nHeight: "+(oBJ.height as string)+"\nCap_Height: "+(oBJ.Cap_Height as string)
						)
						Spindle: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nHeight: "+(oBJ.height as string)+"\nCap_Height: "+(oBJ.Cap_Height as string)+"\nBlend: "+(oBJ.blend as string)
						)
						Gengon: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nHeight: "+(oBJ.height as string)+"\nSides: "+(oBJ.sides as string)+"\nFillet: "+(oBJ.fillet as string)
						)
						C_Ext: (
							oBJtext += "\nBack_Length: "+(oBJ.Back_Length as string)
							oBJtext += "\nSide_Length: "+(oBJ.Side_Length as string)
							oBJtext += "\nFront_Length: "+(oBJ.Front_Length as string)
							oBJtext += "\nBack_Width: "+(oBJ.Back_Width as string)
							oBJtext += "\nSide_Width: "+(oBJ.Side_Width as string)
							oBJtext += "\nFront_Width: "+(oBJ.Front_Width as string)
							oBJtext += "\nHeight: "+(oBJ.height as string)
						)
						L_Ext: (
							oBJtext += "\nSide_Length: "+(oBJ.Side_Length as string)
							oBJtext += "\nFront_Length: "+(oBJ.Front_Length as string)
							oBJtext += "\nSide_Width: "+(oBJ.Side_Width as string)
							oBJtext += "\nFront_Width: "+(oBJ.Front_Width as string)
							oBJtext += "\nHeight: "+(oBJ.height as string)
						)
						Prism: (
							oBJtext += "\nside1Length: "+(oBJ.side1Length as string)
							oBJtext += "\nside2Length: "+(oBJ.side2Length as string)
							oBJtext += "\nside3Length: "+(oBJ.side3Length as string)
							oBJtext += "\nHeight: "+(oBJ.height as string)
						)
						Hedra: (
							local Family
							case oBJ.family of (
								0: (Family = "Tetra")
								1: (Family = "Cube/Octa")
								2: (Family = "Dodec/Icos")
								3: (Family = "Star1")
								4: (Family = "Star2")
							)
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)+"\nFamily: "+Family
						)
						
						-- Poly Meshes
						PolyMeshObject: (
							oBJtext += "\nVerts: "+(oBJ.numverts as string)+"\nFaces: "+(oBJ.numfaces as string)
							oBJtext += getObjectDims oBJ
						)
						Editable_Mesh: (
							oBJtext += "\nVerts: "+(oBJ.numverts as string)+"\nFaces: "+(oBJ.numfaces as string)
							oBJtext += getObjectDims oBJ
						)
						Editable_Poly: (
							oBJtext += "\nVerts: "+(oBJ.numverts as string)+"\nFaces: "+(oBJ.numfaces as string)
							oBJtext += getObjectDims oBJ
						)
					)
					
					gw.text oBJ.Pos oBJtext color:geometryColor
				)
				
				Shape:(
					/** Shape common properties */
					if (ClassOf oBJ != Section) then (
						oBJtext += "\nSteps: "+(oBJ.steps as string)
						oBJtext += "\nOptimize: "+(oBJ.optimize as string)
						oBJtext += "\nAdaptive: "+(oBJ.adaptive as string)
						oBJtext += "\nCurve Length: "+(units.formatValue(curveLength oBJ))
					)

					case ClassOf oBJ of (
						Circle: (
							local diameter = (oBJ.radius * 2)
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nDiameter: "+(diameter as string)
						)
						Arc: (
							oBJtext += "\nRadius: "+(oBJ.radius as string)
						)
						Donut: (
							oBJtext += "\nRadius1: "+(oBJ.radius1 as string)+"\nRadius2: "+(oBJ.radius2 as string)
						)
						Star: (
							oBJtext += "\nRadius1: "+(oBJ.radius1 as string)+"\nRadius2: "+(oBJ.radius2 as string)
						)
						Ellipse: (
							oBJtext += "\nLength: "+(oBJ.length as string)+"\nWidth: "+(oBJ.width as string)
						)
						Egg: (
							oBJtext += "\nLength: "+(oBJ.length as string)+"\nWidth: "+(oBJ.width as string)
						)
						Rectangle: (
							oBJtext += "\nLength: "+(oBJ.length as string)+"\nWidth: "+(oBJ.width as string)+"\nCorner_Radius: "+(oBJ.Corner_Radius as string)
							--oBJtext += "\nLength: "+units.formatValue(oBJ.length)+"\nWidth: "+units.formatValue(oBJ.width)+"\nCorner_Radius: "units.formatValue(oBJ.Corner_Radius)
						)
						Ngon: (
							oBJtext += "\nRadius: "+(oBJ.radius as string)+"\nSides: "+(oBJ.nSides as string)
						)
						Helix: (
							oBJtext += "\nRadius1: "+(oBJ.radius1 as string)+"\nRadius2: "+(oBJ.radius2 as string)+"\nHeight: "+(oBJ.height as string)+"\nTurns: "+(oBJ.turns as string)
						)
						Section: (
							oBJtext += "\nLength: "+(oBJ.length as string)+"\nWidth: "+(oBJ.width as string)
						)
						Text: (
							oBJtext += "\nFont: "+oBJ.font+"\nSize: "+(oBJ.size as string)+"\nText: "+oBJ.text
						)
						Line: (
							oBJtext += getObjectDims oBJ
						)
						SplineShape: (
							oBJtext += getObjectDims oBJ
						)
					)
					
					gw.text oBJ.Pos oBJtext color:shapeColor
				)
			)
		)	--end of for oBJ ...
		
		gw.enlargeUpdateRect #whole
		gw.updateScreen()
	) -- end function showGeometryInfo
	
	on isChecked return AnnotateGeometryEnabled 

	on execute do (
		AnnotateGeometryEnabled  = not AnnotateGeometryEnabled 
		if AnnotateGeometryEnabled then (
			unRegisterRedrawViewsCallback showGeometryInfo
			RegisterRedrawViewsCallback showGeometryInfo
			showGeometryInfo()
		) else (
			unRegisterRedrawViewsCallback showGeometryInfo 
			max views redraw
		)
	)
)

